@@ -1,9 +1,9 @@
 package Net::DNS::Nameserver;
 
-#
-# $Id: Nameserver.pm 1608 2017-12-07 10:10:38Z willem $
-#
-our $VERSION = (qw$LastChangedRevision: 1608 $)[1];
+use strict;
+use warnings;
+
+our $VERSION = (qw$Id: Nameserver.pm 1858 2021-12-08 10:32:12Z willem $)[2];
 
 
 =head1 NAME
@@ -14,12 +14,12 @@ Net::DNS::Nameserver - DNS server class
 
     use Net::DNS::Nameserver;
 
-    my $nameserver = new Net::DNS::Nameserver(
+    my $nameserver = Net::DNS::Nameserver->new(
 	LocalAddr	=> ['::1' , '127.0.0.1'],
 	ZoneFile	=> "filename"
 	);
 
-    my $nameserver = new Net::DNS::Nameserver(
+    my $nameserver = Net::DNS::Nameserver->new(
 	LocalAddr	=> '10.1.2.3',
 	LocalPort	=> 5353,
 	ReplyHandler	=> \&reply_handler
@@ -39,17 +39,9 @@ See L</EXAMPLE> for an example.
 
 =cut
 
-use constant USE_SOCKET_IP => defined eval 'use IO::Socket::IP 0.32; 1;';
+use constant USE_SOCKET_IP => defined eval 'use IO::Socket::IP 0.38; 1;';    ## no critic
+require IO::Socket::INET unless USE_SOCKET_IP;
 
-use constant USE_SOCKET_INET => defined eval 'require IO::Socket::INET';
-
-use constant USE_SOCKET_INET6 => defined eval 'require IO::Socket::INET6';
-
-use constant IPv6 => USE_SOCKET_IP || USE_SOCKET_INET6;
-
-
-use strict;
-use warnings;
 use integer;
 use Carp;
 use Net::DNS;
@@ -61,7 +53,7 @@ use IO::Select;
 use constant FORCE_IPv4 => 0;
 
 use constant DEFAULT_ADDR => qw(::1 127.0.0.1);
-use constant DEFAULT_PORT => 53;
+use constant DEFAULT_PORT => 5353;
 
 use constant STATE_ACCEPTED   => 1;
 use constant STATE_GOT_LENGTH => 2;
@@ -88,7 +80,8 @@ sub new {
 
 	# local server addresses must also be accepted by a resolver
 	my $LocalAddr = $self{LocalAddr} || [DEFAULT_ADDR];
-	my $resolver = new Net::DNS::Resolver( nameservers => $LocalAddr );
+	my $resolver  = Net::DNS::Resolver->new( nameservers => $LocalAddr );
+	$resolver->force_v4(1) unless USE_SOCKET_IP;
 	$resolver->force_v4(1) if FORCE_IPv4;
 	my @localaddresses = $resolver->nameservers;
 
@@ -150,12 +143,12 @@ sub new {
 	# Create the Select object.
 	#--------------------------------------------------------------------------
 
-	my $select = $self{select} = new IO::Select;
+	my $select = $self{select} = IO::Select->new;
 
 	$select->add(@sock_tcp);
 	$select->add(@sock_udp);
 
-	return undef unless $select->count;
+	return unless $select->count;
 
 	#--------------------------------------------------------------------------
 	# Return the object.
@@ -171,7 +164,7 @@ sub new {
 
 sub ReadZoneFile {
 	my ( $self, $file ) = @_;
-	my $zonefile = new Net::DNS::ZoneFile($file);
+	my $zonefile = Net::DNS::ZoneFile->new($file);
 
 	my $RRhash = $self->{RRhash} = {};
 	my $RRlist = [];
@@ -183,6 +176,7 @@ sub ReadZoneFile {
 		if ( $rr->type eq 'SOA' ) { $RRlist = $rr->{RRlist} = [] }
 		else			  { push @$RRlist, $rr }
 	}
+	return;
 }
 
 
@@ -200,10 +194,10 @@ sub ReplyHandler {
 	my ( $leaf, @tail ) = split /\./, $lcase;
 	my $RRhash = $self->{RRhash};
 	my $RRlist = $RRhash->{$leaf} || [];			# hash, then linear search
-	my @match  = grep lc( $_->owner ) eq $lcase, @$RRlist;
+	my @match  = grep { lc( $_->owner ) eq $lcase } @$RRlist;
 
 	if ( $qtype eq 'AXFR' ) {
-		my ($soa) = grep $_->type eq 'SOA', @match;
+		my ($soa) = grep { $_->type eq 'SOA' } @match;
 		if ($soa) { push @ans, $soa, @{$soa->{RRlist}}, $soa }
 		else	  { $rcode = 'NOTAUTH' }
 
@@ -211,14 +205,14 @@ sub ReplyHandler {
 		unless ( scalar(@match) ) {
 			my $wildcard = join '.', '*', @tail;
 			my $wildlist = $RRhash->{'*'} || [];
-			foreach ( grep lc( $_->owner ) eq $wildcard, @$wildlist ) {
+			foreach ( grep { lc( $_->owner ) eq $wildcard } @$wildlist ) {
 				my $clone = bless {%$_}, ref($_);
 				$clone->owner($qname);
 				push @match, $clone;
 			}
 			$rcode = 'NXDOMAIN' unless @match;
 		}
-		@ans = grep $_->type eq $qtype, @match;
+		@ans = grep { $_->type eq $qtype } @match;
 	}
 
 	return ( $rcode, \@ans, [], [], {aa => 1}, {} );
@@ -230,14 +224,7 @@ sub ReplyHandler {
 #------------------------------------------------------------------------------
 
 sub inet_new {
-	return new IO::Socket::INET(@_) unless IPv6;
-
-	return new IO::Socket::IP(@_) if USE_SOCKET_IP;
-
-	my %param = @_;
-
-	return new IO::Socket::INET6(@_) if $param{LocalAddr} =~ /:/;
-	return new IO::Socket::INET(@_);
+	return USE_SOCKET_IP ? IO::Socket::IP->new(@_) : IO::Socket::INET->new(@_);
 }
 
 #------------------------------------------------------------------------------
@@ -249,7 +236,7 @@ sub make_reply {
 
 	unless ($query) {
 		print "ERROR: invalid packet\n" if $self->{Verbose};
-		my $empty = new Net::DNS::Packet();		# create empty reply packet
+		my $empty = Net::DNS::Packet->new();		# create empty reply packet
 		my $reply = $empty->reply();
 		$reply->header->rcode("FORMERR");
 		return $reply;
@@ -314,7 +301,7 @@ sub make_reply {
 
 		if ( !defined($rcode) ) {
 			print "remaining silent\n" if $self->{Verbose};
-			return undef;
+			return;
 		}
 
 		$header->rcode($rcode);
@@ -414,12 +401,14 @@ sub tcp_connection {
 
 			my $qbuf = substr( $self->{_tcp}{$sock}{inbuffer}, 0, $self->{_tcp}{$sock}{querylength} );
 			substr( $self->{_tcp}{$sock}{inbuffer}, 0, $self->{_tcp}{$sock}{querylength} ) = "";
-			my $query = new Net::DNS::Packet( \$qbuf );
+			my $query = Net::DNS::Packet->new( \$qbuf );
 			if ( my $err = $@ ) {
 				print "Error decoding query packet: $err\n" if $self->{Verbose};
 				undef $query;			# force FORMERR reply
 			}
+
 			my $conn = {
+				protocol => $sock->protocol,
 				sockhost => $sock->sockhost,
 				sockport => $sock->sockport,
 				peerhost => $sock->peerhost,
@@ -445,6 +434,7 @@ sub tcp_connection {
 			$self->{_tcp}{$sock}{state} = STATE_SENDING;
 		}
 	}
+	return;
 }
 
 #------------------------------------------------------------------------------
@@ -464,19 +454,20 @@ sub udp_connection {
 		return;
 	}
 
-
 	print "UDP connection from $peerhost:$peerport to $sockhost\n" if $self->{Verbose};
 
-	my $query = new Net::DNS::Packet( \$buf );
+	my $query = Net::DNS::Packet->new( \$buf );
 	if ( my $err = $@ ) {
 		print "Error decoding query packet: $err\n" if $self->{Verbose};
 		undef $query;					# force FORMERR reply
 	}
+
 	my $conn = {
-		sockhost => $sock->sockhost,
+		protocol => $sock->protocol,
 		sockport => $sock->sockport,
-		peerhost => $sock->peerhost,
-		peerport => $sock->peerport
+		sockhost => $sockhost,
+		peerhost => $peerhost,
+		peerport => $peerport
 		};
 	my $reply = $self->make_reply( $query, $peerhost, $conn ) || return;
 
@@ -490,6 +481,7 @@ sub udp_connection {
 	} else {
 		$sock->send( $reply->data($max_len) );
 	}
+	return;
 }
 
 
@@ -556,11 +548,11 @@ sub loop_once {
 
 			# If we have buffered output, then send as much as the OS will accept
 			# and wait with the rest
-			my $len = length $self->{_tcp}{$s}{outbuffer};
-			my $charssent = $sock->syswrite( $self->{_tcp}{$s}{outbuffer} ) || 0;
-			print "Sent $charssent of $len octets to ", $self->{_tcp}{$s}{peer}, ".\n"
+			my $len	 = length $self->{_tcp}{$s}{outbuffer};
+			my $sent = $sock->syswrite( $self->{_tcp}{$s}{outbuffer} ) || 0;
+			print "Sent $sent of $len octets to ", $self->{_tcp}{$s}{peer}, ".\n"
 					if $self->{Verbose};
-			substr( $self->{_tcp}{$s}{outbuffer}, 0, $charssent ) = "";
+			substr( $self->{_tcp}{$s}{outbuffer}, 0, $sent ) = "";
 			if ( length $self->{_tcp}{$s}{outbuffer} == 0 ) {
 				delete $self->{_tcp}{$s}{outbuffer};
 				$self->{_tcp}{$s}{state} = STATE_ACCEPTED;
@@ -596,6 +588,7 @@ sub loop_once {
 			}
 		}
 	}
+	return;
 }
 
 #------------------------------------------------------------------------------
@@ -611,6 +604,7 @@ sub main_loop {
 		# You really need an argument otherwise you'll be burning CPU.
 		$self->loop_once(10);
 	}
+	return;
 }
 
 
@@ -622,12 +616,12 @@ __END__
 
 =head2 new
 
-    $nameserver = new Net::DNS::Nameserver(
+    $nameserver = Net::DNS::Nameserver->new(
 	LocalAddr	=> ['::1' , '127.0.0.1'],
 	ZoneFile	=> "filename"
 	);
 
-    $nameserver = new Net::DNS::Nameserver(
+    $nameserver = Net::DNS::Nameserver->new(
 	LocalAddr	=> '10.1.2.3',
 	LocalPort	=> 5353,
 	ReplyHandler	=> \&reply_handler,
@@ -641,7 +635,7 @@ could not be created.
 Each instance is configured using the following optional arguments:
 
     LocalAddr		IP address on which to listen	Defaults to loopback address
-    LocalPort		Port on which to listen		Defaults to 53
+    LocalPort		Port on which to listen		Defaults to 5353
     ZoneFile		Name of file containing RRs
 			accessed using the default
 			reply-handling subroutine
@@ -694,10 +688,8 @@ See RFC 1035 and the IANA dns-parameters file for more information:
   ftp://ftp.rfc-editor.org/in-notes/rfc1035.txt
   http://www.isi.edu/in-notes/iana/assignments/dns-parameters
 
-The nameserver will listen for both UDP and TCP connections.  On
-Unix-like systems, the program will probably have to run as root
-to listen on the default port, 53.	A non-privileged user should
-be able to listen on ports 1024 and higher.
+The nameserver will listen for both UDP and TCP connections.
+On Unix-like systems, unprivileged users are denied access to ports below 1024.
 
 UDP reply truncation functionality was introduced in VERSION 830.
 The size limit is determined by the EDNS0 size advertised in the query,
@@ -777,7 +769,7 @@ additional filtering on its basis may be applied.
 
 	if ( $qtype eq "A" && $qname eq "foo.example.com" ) {
 		my ( $ttl, $rdata ) = ( 3600, "10.1.2.3" );
-		my $rr = new Net::DNS::RR("$qname $ttl $qclass $qtype $rdata");
+		my $rr = Net::DNS::RR->new("$qname $ttl $qclass $qtype $rdata");
 		push @ans, $rr;
 		$rcode = "NOERROR";
 	} elsif ( $qname eq "foo.example.com" ) {
@@ -797,7 +789,7 @@ additional filtering on its basis may be applied.
     }
 
 
-    my $ns = new Net::DNS::Nameserver(
+    my $ns = Net::DNS::Nameserver->new(
 	LocalPort    => 5353,
 	ReplyHandler => \&reply_handler,
 	Verbose	     => 1
@@ -809,9 +801,9 @@ additional filtering on its basis may be applied.
 
 =head1 BUGS
 
-Limitations in perl 5.8.6 makes it impossible to guarantee that
-replies to UDP queries from Net::DNS::Nameserver are sent from the
-IP-address they were received on. This is a problem for machines with
+Limitations in perl make it impossible to guarantee that replies to
+UDP queries from Net::DNS::Nameserver are sent from the IP-address
+to which the query was directed.  This is a problem for machines with
 multiple IP-addresses and causes violation of RFC2181 section 4.
 Thus a UDP socket created listening to INADDR_ANY (all available
 IP-addresses) will reply not necessarily with the source address being
@@ -844,7 +836,7 @@ All rights reserved.
 
 Permission to use, copy, modify, and distribute this software and its
 documentation for any purpose and without fee is hereby granted, provided
-that the above copyright notice appear in all copies and that both that
+that the original copyright notices appear in all copies and that both
 copyright notice and this permission notice appear in supporting
 documentation, and that the name of the author not be used in advertising
 or publicity pertaining to distribution of the software without specific
